#include "elementBend.h"
#include "mixedTools.h"
#include "mathematicalConstants.h"
#include "PhysicalConstants.h"

elementBend::elementBend() :  abstractElement()
{
  setDefaultValues();
  setDefaults();
  elementName_ = nomdElements("bend");
  nbParam_ = elementName_.getElementNbParameters();
  parametersString_ = new string[nbParam_+1];
  // transportOk_ = true;
  // parmelaOk_ = true;
    registerAcceptableSoftware(nomDeLogiciel::transport, TBoolOk);
    registerAcceptableSoftware(nomDeLogiciel::parmela, TBoolOk);
}

  void elementBend::setDefaultValues()
  {
    defaultLabel_ = "bend";
    angleDegDef_ = 0.0;
    radiusDef_ = 1.0;
    beta1Def_ = 0.0;
    beta2Def_ = 0.0;
    momentumDef_ = 0.0;
  }

  void elementBend::setDefaults()
  {
    label_ = defaultLabel_;
    angleDeg_ = angleDegDef_;
    radius_ = radiusDef_;
    beta1_ = beta1Def_;
    beta2_ = beta2Def_;
    momentum_ = momentumDef_;
  }

 string* elementBend::getParametersString() const 
 {
   int compteur = -1;
   parametersString_[++compteur] = mixedTools::intToString(nbParam_);
   parametersString_[++compteur] = label_;
   parametersString_[++compteur] = mixedTools::doubleToString( angleDeg_);
   parametersString_[++compteur] = mixedTools::doubleToString(radius_);
   parametersString_[++compteur] = mixedTools::doubleToString(momentum_);
   parametersString_[++compteur] = mixedTools::doubleToString(aperture_);
   parametersString_[++compteur] = mixedTools::doubleToString(beta1_);
   parametersString_[++compteur] = mixedTools::doubleToString(beta2_);
   if ( compteur != nbParam_ ) {
     cerr << " elementBend::getParametersString() : ERROR nr pf parameters dosnt match " << endl;
     return NULL;
   }
   return parametersString_;
 }

 double elementBend::getLenghtOfElement() const 
{
 return 1.0e2 * radius_ * angleDeg_ * DEG_TO_RADIANS; 
}


 void  elementBend::setParametersString(string* param)
 {
  	if ( param == NULL )
	{
	  cerr << "  elementBend::setParametersString parameters empty parameter set";
	  return;
	}

   int compteur = -1;
   int nbparam = atoi(param[++compteur].c_str());
    if ( nbparam != nbParam_ ) 
	{
	  cerr << "  elementBend::setParametersString parameters do not match for a BEND" << endl;
	  return;
	}
    label_ = param[++compteur];
    angleDeg_ =  atof(param[++compteur].c_str());
    radius_ = atof(param[++compteur].c_str());
    momentum_ = atof(param[++compteur].c_str());
    aperture_  = atof(param[++compteur].c_str());
    beta1_ = atof(param[++compteur].c_str());
    beta2_ = atof(param[++compteur].c_str());
    // calcul de la longueur : rayon en m ; longueur en cm
    lenghtElem_ = getLenghtOfElement();
    cout << " elementBend::setParametersString calcule longueur = " << lenghtElem_ << endl;
 }

string elementBend::parmelaOutputFlow() const
  {
    ostringstream sortie;
    // il faut entrer l'energie cinetique
    double ecin = momentum_/EREST_MeV;
    ecin = ecin*ecin + 1.;
    ecin = EREST_MeV*(sqrt(ecin) - 1.0);
    sortie << "BEND /l=" << lenghtElem_ << "  / aper=" << aperture_ << "  / iout=1 / wr=" << ecin << " /alpha=" << angleDeg_ << " / beta1=" << beta1_;
    sortie << " / beta2=" << beta1_;
    return sortie.str();
  }


string elementBend::transportOutputFlow() const
  {
    ostringstream sortie;
    sortie << label_ << ":" << "  SBEND, RADIUS=" << radius_ << ", ANGLE=" << angleDeg_ << ", E1=" << beta1_ << ", E2=" << beta2_ << ";";
    return sortie.str();
  }


string elementBend::FileOutputFlow() const
  {
    ostringstream sortie;
    //    sortie << elementName_.getElementType() << endl;
    sortie << elementName_.getElementLabel() << endl;
    sortie  << label_ << endl;
    sortie << angleDeg_ << "  " << radius_ << " " <<  aperture_  << endl;
    sortie << beta1_ << "  " << beta2_ << " " << momentum_ << endl;
    return sortie.str();
  }

void elementBend::FileInput(ifstream& ifs)
{
  ifs >> label_;
  ifs >> angleDeg_ >> radius_ >>  aperture_;
  ifs >> beta1_ >> beta2_ >> momentum_;
    lenghtElem_ = getLenghtOfElement();
    cout << " elementBend::FileInput calcule longueur = " << lenghtElem_ << endl;
}


string elementBend::print() {
    string txt = "";

    txt += label_;
    txt += "\nangle (deg.) : ";
    txt += mixedTools::doubleToString(angleDeg_);
    txt += "\nradius (m) : ";
    txt += mixedTools::doubleToString(radius_);
    txt += "\nmomentum (MeV/c) : ";
    txt += mixedTools::doubleToString(momentum_);
    txt += "\nlength (cm) : ";
    txt += mixedTools::doubleToString(lenghtElem_);
    txt += "\naperture (cm) : ";
    txt += mixedTools::doubleToString(aperture_);
    txt += "\npole face rotations (deg.) entrance : ";
    txt += mixedTools::doubleToString(beta1_);
    txt += "\nexit : ";
    txt += mixedTools::doubleToString(beta2_);
    
    return txt;
}
