#include <iostream>

#include "GWt_elementLigneFaisceau.h"
#include "nomdElements.h"
#include "GWt_console.h"
#include "abstractElement.h"
#include "GWt_rfgun.h"
#include "GWt_drift.h"
#include "GWt_cell.h"
#include "GWt_bend.h"
#include "GWt_soleno.h"
#include "GWt_beam.h"
#include "GWt_fit.h"
#include "GWt_snapshot.h"


#include <Wt/WCssDecorationStyle>
#include <Wt/WSignalMapper>
#include <Wt/WText>
#include <Wt/WImage>
#include <Wt/WHBoxLayout>
#include <Wt/WGridLayout>
#include <Wt/WLength>
#include <Wt/WVBoxLayout>
#include <Wt/WApplication>

#define PSPA_DEBUG 1

GWt_elementLigneFaisceau::GWt_elementLigneFaisceau(PspaApplication* pspa)
: WContainerWidget(),
firstTimeInsertMessage_(NULL),
dropZoneLeft_(NULL),
dropZoneRight_(NULL),
GWt_abstractElement_(NULL),
elementLabel_(NULL),
pspa_(pspa)
{
    initialize_ligneFaisceau(inconnu);
}


GWt_elementLigneFaisceau::GWt_elementLigneFaisceau(typedElement type, PspaApplication* pspa)
: WContainerWidget(),
firstTimeInsertMessage_(NULL),
dropZoneLeft_(NULL),
dropZoneRight_(NULL),
GWt_abstractElement_(NULL),
elementLabel_(NULL),
pspa_(pspa)
{
    // set the css name
    setObjectName("elementLigneFaisceau");
    initialize_ligneFaisceau(type);
}

void GWt_elementLigneFaisceau::initialize_ligneFaisceau(typedElement eType)
{
    
    cout<<"new GWt_elementLigneFaisceau : "<< this << endl;
    
    // set layout
    //     +----------+--------+---------+
    //     |   (--->) |  ELEM  | (-----) |
    //     +----------+--------+---------+
    
    
    WGridLayout *gridLayout = new WGridLayout();
    
    gridLayout->setContentsMargins(0,0,0,0);
    gridLayout->setHorizontalSpacing(0);
    gridLayout->setVerticalSpacing(0);
    
    // add first line :  initialize arrow and link images
    dropZoneLeft_ = new GWt_dropZoneLigneFaisceau(pspa_);
    dropZoneRight_ = new GWt_dropZoneLigneFaisceau(pspa_);
    
    // change object name in order to manage css :
    dropZoneLeft_->resize(10,30);
    dropZoneRight_->resize(10,30);
    
    gridLayout->addWidget	(dropZoneLeft_,0,0);
    gridLayout->addWidget	(dropZoneRight_,0,2);
    
    // add first line :  initialize arrow and link images
    dropZoneLeft_ = new GWt_dropZoneLigneFaisceau(pspa_);
    dropZoneRight_ = new GWt_dropZoneLigneFaisceau(pspa_);
    
    // change object name in order to manage css :
    dropZoneLeft_->resize(10,30);
    dropZoneRight_->resize(10,30);
    
    gridLayout->addWidget	(dropZoneLeft_,0,0);
    gridLayout->addWidget	(dropZoneRight_,0,2);
    
    // add the image if present
    
    switch (  eType ) {
        case RFgun :
            GWt_abstractElement_ = new GWt_rfgun(this);
            break;
        case drift :
            GWt_abstractElement_ = new GWt_drift(this);
            break;
        case cell  :
            GWt_abstractElement_ = new GWt_cell(this);
            break;
        case bend  :
            GWt_abstractElement_ = new GWt_bend(this);
            break;
        case soleno  :
            GWt_abstractElement_ = new GWt_soleno(this);
            break;
        case beam  :
            GWt_abstractElement_ = new GWt_beam(this);
            break;
        case fit  :
            GWt_abstractElement_ = new GWt_fit(this);
            break;
        case inconnu :
            GWt_abstractElement_ = NULL;
    }
    
    elementContainer_ = new WContainerWidget();
    WVBoxLayout* vLayoutElementContainer = new WVBoxLayout();
    
    elementContainer_->setStyleClass("beamLineIcon");
    elementContainer_->setLayout(vLayoutElementContainer);
    
    
    
    if (GWt_abstractElement_ != NULL) {
        vLayoutElementContainer->addWidget(GWt_abstractElement_);
        GWt_abstractElement_->setMaximumSize(80,80);
        GWt_abstractElement_->setMinimumSize(80,80);
    }
    
    if (GWt_abstractElement_ == NULL) {
        elementLabel_ = new WText();
    } else {
        elementLabel_ = new WText(GWt_abstractElement_->getAbstractElement()->getLabel());
    }
    
    vLayoutElementContainer->addWidget(elementLabel_,Wt::AlignCenter |Wt::AlignMiddle);
    //    gridLayout->addWidget(elementLabel_,1,0,1,3,Wt::AlignCenter |Wt::AlignMiddle);
    
    // Add Actions
    WContainerWidget* actions = new WContainerWidget();
    WVBoxLayout* vLayoutActions = new WVBoxLayout();
    actions->setLayout(vLayoutActions);
    
    WText* actionParameters = new WText("&nbsp;");
    WText* actionSoftware = new WText("Software");
    
    // Set class for actions elements
    actionParameters->setStyleClass("Button WhiteButton ContrastButton parameters");
    actionParameters->setMaximumSize(32,32);
    actionParameters->setMinimumSize(32,32);
    
    actionSoftware->setStyleClass("Button WhiteButton ContrastButton software");
    actionParameters->setPositionScheme(Wt::Static);
    
    // Set the global action class
    actions->setStyleClass("beamLineElementActions");
    actions->setPositionScheme(Wt::Static);
    actions->setOffsets(0);
    
    vLayoutActions->addWidget(actionParameters);
    vLayoutActions->addWidget(actionSoftware);
    vLayoutElementContainer->addWidget(actions);
    
    gridLayout->addWidget	(elementContainer_,0,1);
    
    gridLayout->setColumnStretch (0,1);
    gridLayout->setColumnStretch (2,1);
    
    // set layout
    setLayout(gridLayout);
    
}


GWt_elementLigneFaisceau::~GWt_elementLigneFaisceau()
{
}

void GWt_elementLigneFaisceau::setBGColor(const WColor c) {
    elementContainer_->decorationStyle().setBackgroundColor (c);
}


void GWt_elementLigneFaisceau::removeDone(GWt_abstractElement* gw)
{
    if (messageDropZone_->result() == WDialog::Accepted) {
        // FIXME
        //    removeDropZone();
    }
}


void GWt_elementLigneFaisceau::addElement(GWt_abstractElement* elem)
{
    clear();
    GWt_abstractElement_ = elem;
    addWidget(GWt_abstractElement_);
    if (static_cast <WContainerWidget*> (parent())) {
        WContainerWidget* wc = static_cast <WContainerWidget*> (parent());
        WBoxLayout* layout = ((WBoxLayout*)(wc->layout()));
        layout->setStretchFactor(this,0);
    }
    
    
    firstTimeInsertMessage_ = NULL;
    dropZoneLeft_ = NULL;
}


std::string GWt_elementLigneFaisceau::getElementNumberInBeamLine(nomdElements elem){
    string str = "";
    
    if (static_cast <WContainerWidget*> (parent())) {
        WContainerWidget* wc = static_cast <WContainerWidget*> (parent());
        for (int a=0; a<wc->count(); a++) {
            if (static_cast <GWt_abstractElement*> (wc->widget (a))) {
                //        GWt_abstractElement* ab = static_cast <GWt_abstractElement*> (wc->widget (a));
                /*
                 printf("1--->%d\n",ab->getElement());
                 printf("2--->%d\n",ab->getElement()->getLenghtOfElement());
                 printf("3--->%d\n",ab->getElement()->getName().getElementType());
                 printf("4--->%d\n",elem.getElementType());
                 
                 if (static_cast <GWt_abstractElement*> (wc->widget (a))->getElement()->getName().getElementType() == elem.getElementType()) {
                 pspa_->addConsoleMessage("type trouvÈ");
                 } else {
                 pspa_->addConsoleMessage("type trouvÈ NOT ");
                 }
                 */
                
            }
        }
        
        int n = 999999;
        
        std::stringstream stream;
        stream << n ;
        std::string number(stream.str());
        
        if(n < 10) {
            str= elem.getElementLabel()+"0"+number;
        } else {
            str= elem.getElementLabel()+number;
        }
    }
    
    return str;
}


void GWt_elementLigneFaisceau::setElementLabel(std::string label) {
    if (elementLabel_) {
        elementLabel_->setText(label.substr(0,10));
        elementLabel_->setToolTip(label);
    }
}
